/*

*/
#include <windows.h>
#include "glut.h"

// Prototypes for glut callback functions
void display( void );
void reshape(int, int);
void keyboard(unsigned char, int, int);

int windowHeight;  // this is set by the glut reshape() callback function
int windowWidth;   // this is set by the glut reshape() callback function


int main(int argc, char** argv)
{
   // Standard GLUT initialization
   glutInit(&argc,argv);
   glutInitDisplayMode (GLUT_SINGLE | GLUT_RGB | GLUT_DEPTH);
   glutInitWindowSize(500, 500);
   glutInitWindowPosition(150, 150);
   glutCreateWindow("Depth Test");
   glutDisplayFunc(display);
   glutReshapeFunc(reshape);
   glutKeyboardFunc(keyboard);

   // Initialize OpenGL state
   glClearColor(1.0, 1.0, 1.0, 1.0); // white background
   glEnable(GL_DEPTH_TEST);

   glFrontFace(GL_CCW);
   glPolygonMode(GL_FRONT, GL_FILL);
   glPolygonMode(GL_BACK, GL_FILL);

   printf("\nCS 45500 Depth Testing Example:\n");
   printf("Use 'd' and 'D' keys to turn on and off depth testing.\n");
   printf("\n");
   printf("Depth testing is turned on.\n");

   glutMainLoop(); // Enter event loop
   return 0;       // We never get here
}//main()


// The display() callback function
void display()
{
   glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);  // clear the window

   // specify the viewport
   glViewport(0, 0, windowWidth, windowHeight); // left, bottom, width, height (in pixels)

   // set up viewing
   glMatrixMode(GL_PROJECTION);
   glLoadIdentity();
   //glFrustum(-1.0, 1.0, -1.0, 1.0, 1.0, 4.0);
   glOrtho(-1.0, 1.0, -1.0, 1.0, 1.0, 4.0);

   glMatrixMode(GL_MODELVIEW);
   glLoadIdentity();
   gluLookAt(1.0, 1.0, 2.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0);

   // draw a vertical, filled in polygon in the middle of the cube

   glBegin(GL_QUADS);
     glColor3ub(255, 0, 0);      // draw in red
     glVertex3f( 0.5,  0.5, 1.0);
     glVertex3f(-0.5,  0.5, 1.0);
     glVertex3f(-0.5, -0.5, 1.0);
     glVertex3f( 0.5, -0.5, 1.0);
     glColor3ub(0, 255, 0);      // draw in green
     glVertex3f( 0.5,  0.5, -0.5);
     glVertex3f(-0.5,  0.5, -0.5);
     glVertex3f(-0.5, -0.5, -0.5);
     glVertex3f( 0.5, -0.5, -0.5);
     glColor3ub(0, 0, 255);      // draw in blue
     glVertex3f( 0.0,  0.25,  1.5);
     glVertex3f( 0.0,  0.25, -1.5);
     glVertex3f( 0.0, -0.25, -1.5);
     glVertex3f( 0.0, -0.25,  1.5);
   glEnd();

   //printf("We got a \"display()\" event\n");
   glFlush(); // clear buffers
}//display()



// The reshape() callback function
void reshape(int width, int height)
{
   windowWidth  = width;
   windowHeight = height;

   //printf("We got a \"reshape(%d, %d)\" event\n", width, height);

   glutPostRedisplay();
}//reshape()


/* glut keyboard() call back function */
void keyboard(unsigned char key, int x, int y)
{
   switch (key)
   {
      case 'd':
         glEnable(GL_DEPTH_TEST);
         printf("Depth testing is turned on.\n");
         break;
      case 'D':
         glDisable(GL_DEPTH_TEST);
         printf("Depth testing is turned off.\n");
         break;
      default:
         break;
   }
   glutPostRedisplay();
}//keyboard()
