/*

*/
#include <windows.h>
#include "glut.h"

/* Prototypes for the GLUT callback functions */
void display( void );
void keyboard( unsigned char, int, int );


void display(void)
{
   /* Polished Gold material properties*/
   GLfloat mat_f_ambient[]   = { 0.24725,  0.2245,   0.0645,   1.0 };
   GLfloat mat_f_diffuse[]   = { 0.34615,  0.3143,   0.0903,   1.0 };
   GLfloat mat_f_specular[]  = { 0.797357, 0.723991, 0.208006, 1.0 };
   GLfloat mat_f_shininess[] = { 83.2 };
   /* Jade material properties*/
   GLfloat mat_b_ambient[]   = { 0.135,    0.2225,   0.1575,   0.95 };
   GLfloat mat_b_diffuse[]   = { 0.54,     0.89,     0.63,     0.95 };
   GLfloat mat_b_specular[]  = { 0.316228, 0.316228, 0.316228, 0.95 };
   GLfloat mat_b_shininess[] = { 12.8 };

   GLfloat light_position[] = { 0.0, 2.0, 0.0, 0.0 };
   GLfloat light_ambient[]  = { 0.5, 0.5, 0.5, 0.0 };
   GLfloat light_diffuse[]  = { 0.5, 0.5, 0.5, 0.0 };
   GLfloat light_specular[] = { 0.5, 0.5, 0.5, 0.0 };
   GLfloat global_ambient[] = { 0.5, 0.5, 0.5, 0.0 };  // white light

   glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

   glMatrixMode(GL_PROJECTION);
   glLoadIdentity();
   glFrustum(-3.0, 3.0, -3.0, 3.0, 1.0, 20.0);

   glMatrixMode(GL_MODELVIEW);
   glLoadIdentity();
   gluLookAt(0.0, 2.0, 3.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0);

   /* Set the properties of Light 0 */
   glLightfv(GL_LIGHT0, GL_POSITION, light_position);
   glLightfv(GL_LIGHT0, GL_AMBIENT,  light_ambient);
   glLightfv(GL_LIGHT0, GL_DIFFUSE,  light_diffuse);
   glLightfv(GL_LIGHT0, GL_SPECULAR, light_specular);
   glLightf(GL_LIGHT0, GL_CONSTANT_ATTENUATION,  1.0);
   glLightf(GL_LIGHT0, GL_LINEAR_ATTENUATION,    1.0);
   glLightf(GL_LIGHT0, GL_QUADRATIC_ATTENUATION, 1.0);

   /* Set global light properties */
   glLightModeli(GL_LIGHT_MODEL_LOCAL_VIEWER, GL_TRUE);
   glLightModeli(GL_LIGHT_MODEL_TWO_SIDE, GL_TRUE);
   glLightModelfv(GL_LIGHT_MODEL_AMBIENT, global_ambient ); // white ambient light

   /* Set the material properties */
   glMaterialfv(GL_FRONT, GL_AMBIENT,   mat_f_ambient);
   glMaterialfv(GL_FRONT, GL_DIFFUSE,   mat_f_diffuse);
   glMaterialfv(GL_FRONT, GL_SPECULAR,  mat_f_specular);
   glMaterialfv(GL_FRONT, GL_SHININESS, mat_f_shininess);
   glMaterialfv(GL_BACK, GL_AMBIENT,   mat_b_ambient);
   glMaterialfv(GL_BACK, GL_DIFFUSE,   mat_b_diffuse);
   glMaterialfv(GL_BACK, GL_SPECULAR,  mat_b_specular);
   glMaterialfv(GL_BACK, GL_SHININESS, mat_b_shininess);

   /* Draw a (lighted) object */
   glBegin(GL_POLYGON);
      glNormal3f(0.0, 1.0, 0.0);
      glVertex3f( 2.0, 0.0, -2.0);
      glVertex3f(-2.0, 0.0, -2.0);
      glNormal3f(1.0, 1.0, 1.0);  // put a different normal at the opposite corner
      glVertex3f(-2.0, 0.0,  2.0);
      glVertex3f( 2.0, 0.0,  2.0);
   glEnd();

   glFlush ();
}



int main(int argc, char** argv)
{
   glutInit(&argc, argv);
   glutInitDisplayMode(GLUT_SINGLE | GLUT_RGB | GLUT_DEPTH);
   glutInitWindowSize(500, 500);
   glutInitWindowPosition(100, 100);
   glutCreateWindow(argv[0]);
   glClearColor(0.0, 0.0, 0.0, 0.0);

   glShadeModel(GL_SMOOTH);  // this is the OpenGL default value
   glEnable(GL_LIGHTING);
   glEnable(GL_LIGHT0);

   glutDisplayFunc(display);
   glutKeyboardFunc(keyboard);
   glutMainLoop();
   return 0;
}


void keyboard(unsigned char key, int x, int y)
{
   switch (key)
   {
      case 'f':
         glShadeModel(GL_FLAT);
         break;
      case 's':
         glShadeModel(GL_SMOOTH);
         break;
      default:
         break;
   }
   glutPostRedisplay();
}//keyboard()